@@ -54,6 +54,7 @@ import java.io.IOException;
 import java.nio.charset.Charset;
 
 import com.fasterxml.jackson.databind.ObjectMapper;
+import com.openexchange.java.Strings;
 import com.openexchange.mobile.api.facade.auth.LoginCredentials;
 import com.openexchange.mobile.api.facade.auth.SessionData;
 import com.openexchange.mobile.api.facade.connectors.Connector;
@@ -61,7 +62,6 @@ import com.openexchange.mobile.api.facade.connectors.RequestConfiguration;
 import com.openexchange.mobile.api.facade.exceptions.ApiFacadeException;
 import com.openexchange.mobile.api.facade.utils.JacksonUtil;
 import com.openexchange.mobile.api.facade.utils.RequestUtil;
-import com.openexchange.mobile.api.facade.utils.StringUtils;
 
 import lombok.extern.slf4j.Slf4j;
 
@@ -72,6 +72,7 @@ import org.apache.http.client.CookieStore;
 import org.apache.http.client.methods.CloseableHttpResponse;
 import org.apache.http.client.methods.HttpUriRequest;
 import org.apache.http.client.methods.RequestBuilder;
+import org.apache.http.conn.ConnectTimeoutException;
 import org.apache.http.entity.BufferedHttpEntity;
 import org.apache.http.impl.client.CloseableHttpClient;
 import org.apache.http.util.EntityUtils;
@@ -103,33 +104,47 @@ public class LoginConnector implements Connector<SessionData, LoginResponseMto>
             if (responseBody.isError()) {
                 throw ApiFacadeException.fromResponseBody(responseBody);
             }
-            if (StringUtils.isEmpty(responseBody.getSession())) {
+            if (Strings.isEmpty(responseBody.getSession())) {
                 throw ApiFacadeException.internalServerError("No session received.");
             }
             if (cookieStore.getCookies().size() == 0) {
                 throw ApiFacadeException.internalServerError("No cookies received.");
             }
-            return new SessionData(responseBody.getSession(), cookieStore.getCookies());
+            SessionData sessionData = new SessionData(responseBody.getSession(), cookieStore.getCookies());
+            sessionData.setProperty(SessionData.PROPERTY_CONTEXT_ID, Long.toString(responseBody.getContextId()));
+            sessionData.setProperty(SessionData.PROPERTY_USER_ID, Long.toString(responseBody.getUserId()));
+            return sessionData;
         } catch (Exception e) {
+            if (e instanceof ConnectTimeoutException) {
+                throw ApiFacadeException.gatewayTimeoutError(e);
+            }
             throw ApiFacadeException.internalServerError(e);
         }
     }
 
     private HttpUriRequest createRequest() {
         String clientId = "open-xchange-mobile-api-facade";
-        if (!StringUtils.isEmpty(credentials.getClientId())) {
+        if (!Strings.isEmpty(credentials.getClientId())) {
             clientId += "-" + credentials.getClientId();
         }
         RequestBuilder builder = RequestBuilder.post(createUrl());
         builder.setCharset(Charset.forName("UTF-8"));
+        String hostname = credentials.getHost();
+        if (!Strings.isEmpty(hostname)) {
+            builder.addHeader(RequestUtil.HEADER_HOST, hostname);
+        }
         String forwardedFor = credentials.getForwardedFor();
-        if (!StringUtils.isEmpty(forwardedFor)) {
+        if (!Strings.isEmpty(forwardedFor)) {
             builder.addHeader(RequestUtil.HEADER_X_FORWARDED_FOR, forwardedFor);
         }
         String userAgent = credentials.getUserAgent();
         if (userAgent != null) {
             builder.addHeader(RequestUtil.HEADER_USER_AGENT, userAgent);
         }
+        String trackingId = credentials.getTrackingId();
+        if (trackingId != null) {
+            builder.addParameter("trackingId", trackingId);
+        }
         return builder
                 .addParameter("action", "login")
                 .addParameter("name", credentials.getUsername())
